
/* 
   Project:     SOFTSUSY 1.2
   Date:        18.07.2001
   File:        interface.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: interface from fortran calling program to rest of code

   $Log: interface.cpp,v $
   Revision 1.3  2001/07/30 15:11:14  allanach
   *** empty log message ***

   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#include "complex.h"
#include "def.h"
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"
// Convert C to fortran naming convention
#define interfacesugra interfacesugra_

// User supplied routine. Inputs m at the unification scale, and uses
// inputParameters vector to output m with high energy soft boundary
// conditions. 
void sugraBcs(MssmSoftsusy & m, const DoubleVector & inputParameters)
{
  double m0 = inputParameters.display(1);
  double m12 = inputParameters.display(2);
  double a0 = inputParameters.display(3);

  // Sets scalar soft masses equal to m0, fermion ones to m12 and sets the
  // trilinear scalar coupling to be a0
  m.standardSugra(m0, m12, a0);
    
  return;
}

// outputs vector for use in above routine given SUGRA parameters 
// m0,m12,a0
void translateSugra(DoubleVector & pars, double m0, double m12,
			    double a0) 
{
  pars(1) = m0; pars(2) = m12; pars(3) = a0;
}

static int beenHereBefore;

/*extern "C" void interfacesugra(double *m0, double *m12, double *a0, double
			       *mgut, double *tanb, int *signMu, double
			       *runningParameters, double *physicalParameters) */
extern "C" void interfacesugra(double *m0, double *m12, double *a0, double
			       *mgut, double *tanb, int *signMu, double
			       *runningParameters, double *physicalParameters,
			       double *fineTuningMeasure) 
{
 // Sets format of output: 2 decimal places
  outputCharacteristics(4);

  if (beenHereBefore == 0) {
    cout << "\n\n\nSOFTSUSY test program, Ben Allanach 2001.\n";
    cout << "If you use SOFTSUSY, please refer to hep-ph/0104145\n\n\n";
  }

  int accuracy = 3; // accuracy = 3 implies all loop/finite corrections will
                    // be used 

  // initialise the data from the input file "massIn"
  QedQcd oneset;
  readIn(oneset, "massIn");

  if (beenHereBefore == 0) cout << "Data input:\n" << oneset << endl;

  // run the quark masses and QED,QCD gauge couplings (to 1 and 3 loops
  // respectively) up to MZ
  oneset.toMz();

  DoubleVector pars(3); translateSugra(pars, *m0, *m12, *a0);

  // Return r as an MssmSoftsusy object consistent with unification 
  // given by sugraBcs at mgut and the low energy data in oneset to 
  // accuracy epsilon
  const double epsilon = 1.0e-5;
 
  MssmSoftsusy r;
  r.lowOrg(sugraBcs, *mgut, pars, *signMu, *tanb, oneset, accuracy,
	   epsilon);

  // If there's a problem, eg lack of EWSB, print this to the screen
  if (r.displayProblem().test()) 
    cout << "Problems with point: " << r.displayProblem() << flush;

  // Calculate fine tuning with respect to parameters given in pars (and
  // mu,B,ht)
  DoubleVector ft(6); ft = r.fineTune(sugraBcs, pars, *mgut);  
  
  // Transfer output to vectors to pass to fortran
  r.display().display(runningParameters);
  r.displayPhys().display(physicalParameters);
  ft.display(fineTuningMeasure);

  beenHereBefore = 1;

  // Interfaces to Monte Carlo codes:
  cout << "************** ISAWIG input **************\n";
  r.isawigInterface751("softsusy.out", "softsusy.in");
  //cout << "\n\n************** ISAJET 7.51 par file input **************\n";
  //r.isajetInterface751();
  //cout << "\n\n************** SSRUN input **************\n";
  //r.ssrunInterface751("softsusy.out");
  //cout << endl << endl;
}
