
/* 
   Project:     SOFTSUSY 1.2
   File:        main.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: main calling program example

   $Log: main.cpp,v $
   Revision 1.5  2001/07/30 15:11:14  allanach
   *** empty log message ***

   Revision 1.4  2001/07/30 14:08:47  allanach
   Added ISAWIG and SSRUN interface

   Revision 1.3  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#include "iostream.h"
#include "complex.h"
#include "def.h"
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"

// User supplied routine. Inputs m at the unification scale, and uses
// inputParameters vector to output m with high energy soft boundary
// conditions. 
void sugraBcs(MssmSoftsusy & m, const DoubleVector & inputParameters)
{
  double m0 = inputParameters.display(1);
  double m12 = inputParameters.display(2);
  double a0 = inputParameters.display(3);

  // Sets scalar soft masses equal to m0, fermion ones to m12 and sets the
  // trilinear scalar coupling to be a0
  m.standardSugra(m0, m12, a0);
    
  return;
}

void userDefinedBcs(MssmSoftsusy & m, const DoubleVector & inputParameters) {
  m.methodBoundaryCondition(inputParameters);
  sugraBcs(m, inputParameters);
}

// outputs vector for use in above routine given SUGRA parameters 
// m0,m12,a0
void translateSugra(DoubleVector & pars, double m0, double m12,
			    double a0) {
  pars(1) = m0; pars(2) = m12; pars(3) = a0;
}

int main() {
 // Sets format of output: 2 decimal places
  outputCharacteristics(4);

  cout << "SOFTSUSY1.2 test program, Ben Allanach 2001\n";
  cout << "If you use SOFTSUSY, please refer to hep-ph/0104145\n\n";

  // Parameters used
  double m12 = 400., a0 = 0., mgut = 1.9e16, tanb = 10.0, m0 = 400.;
  int sgnMu = 1, accuracy = 3; // accuracy = 3 implies all loop/finite
                               // corrections will be used

  QedQcd oneset;
  readIn(oneset, "massIn");

  cout << "Low energy data:\n" << oneset;

  oneset.toMz();

  DoubleVector pars(3); 

  // Return r as an MssmSoftsusy object consistent with unification 
  // given by sugraBcs at mgut and the low energy data in oneset to 
  // accuracy epsilon
  const double epsilon = EPS;

  MssmSoftsusy r;   
        
  translateSugra(pars, m0, m12, a0);
  r.lowOrg(sugraBcs, mgut, pars, sgnMu, tanb, oneset, accuracy, epsilon);

  // Interfaces to Monte Carlo codes:
  //cout << "************** ISAWIG input **************\n";
  //r.isawigInterface751("softsusy.out", "softsusy.in");
  //cout << "\n\n************** ISAJET 7.51 par file input **************\n";
  //r.isajetInterface751();
  cout << "\n\n************** ISASUSY input **************\n";
  r.ssrunInterface751("softsusy.out");
  cout << endl << endl;

  cout << r << endl; 

  cout << "******** Fine Tuning wrt (m0,m12,a0,mu,B,ht) **********" << endl;
  if (!r.displayProblem().test()) cout << r.fineTune(sugraBcs, pars, mgut);
}
